assert( oRA, "oRA not found!")

------------------------------
--      Are you local?      --
------------------------------

local L = AceLibrary("AceLocale-2.0"):new("oRAOMainTank")
local roster = AceLibrary("RosterLib-2.0")

local bartextures = {
	["default"]     = "Interface\\TargetingFrame\\UI-StatusBar",
	["otravi"]      = "Interface\\AddOns\\oRA2\\Textures\\otravi",
	["perl"]        = "Interface\\AddOns\\oRA2\\Textures\\perl",
	["smooth"]      = "Interface\\AddOns\\oRA2\\Textures\\smooth",
	["striped"]     = "Interface\\AddOns\\oRA2\\Textures\\striped",
	["bantobar"]	= "Interface\\AddOns\\oRA2\\Textures\\bantobar",
}

----------------------------
--      Localization      --
----------------------------

L:RegisterTranslations("enUS", function() return {
	["maintankoptional"] = true,
	["mt"] = true,
	["MainTank"] = true,
	["Optional/MainTank"] = true,
	["Options for the maintanks."] = true,
	["Bartexture"] = true,
	["default"] = true,
	["otravi"] = true,
	["perl"] = true,
	["smooth"] = true,
	["striped"] = true,
	["bantobar"] = true,
	["Set statusbar textures."] = true,
	["Targettarget"] = true,
	["Toggle TargetTarget frames."] = true,
	["Scale"] = true,
	["Set frame scale."] = true,
	["Alpha"] = true,
	["Set frame alpha."] = true,
	["Raidicon"] = true,
	["Toggle raid icons."] = true,
	["Frames"] = true,
	["Options for the maintank frames."] = true,
	["Growup"] = true,
	["Toggle growup."] = true,
	["Inverse"] = true,
	["Toggle inverse healthbar."] = true,
	["Deficit"] = true,
	["Toggle deficit health."] = true,
	["Clickcast"] = true,
	["Toggle clickcast support."] = true,
} end )

----------------------------------
--      Module Declaration      --
----------------------------------

oRAOMainTank = oRA:NewModule(L["maintankoptional"])
oRAOMainTank.defaults = {
	bartexture = L["default"],
	targettarget = true,
	raidicon = true,
	alpha = 1,
	scale = 1,
	growup = false,
	inverse = false,
	deficit = false,
	clickcast = false,
}
oRAOMainTank.optional = true
oRAOMainTank.name = L["Optional/MainTank"]
oRAOMainTank.consoleCmd = L["mt"]
oRAOMainTank.consoleOptions = {
	type = "group",
	desc = L["Options for the maintanks."],
	name = L["MainTank"],
	args = {
		[L["Frames"]] = {
			type = "group",
			desc = L["Options for the maintank frames."],
			name = L["Frames"],
			args = {
				[L["Bartexture"]] = {
					type = "text",
					name = L["Bartexture"],
					desc = L["Set statusbar textures."],
					get = function() return oRAOMainTank.db.profile.bartexture end,
					set = function(v) oRAOMainTank:SetBarTexture(v)	end,
					validate = { L["bantobar"], L["smooth"], L["otravi"], L["perl"], L["striped"], L["default"] }
				},
		
				[L["Targettarget"]] = {
					type = "toggle",
					name = L["Targettarget"],
					desc = L["Toggle TargetTarget frames."],
					get = function() return oRAOMainTank.db.profile.targettarget end,
					set = function(v) oRAOMainTank:ToggleTargetTarget(v) end,
				},
		
				[L["Scale"]] = {
					type = "range",
					name = L["Scale"],
					desc = L["Set frame scale."],
					get = function() return oRAOMainTank.db.profile.scale end,
					set = function(v) oRAOMainTank:SetScale(v) end,
					min = 0.1,
					max = 2,
				},				

				[L["Alpha"]] = {
					type = "range",
					name = L["Alpha"],
					desc = L["Set frame alpha."],
					get = function() return oRAOMainTank.db.profile.alpha end,
					set = function(v) oRAOMainTank:SetAlpha(v) end,
					min = 0.1,
					max = 1,
				},
	
				[L["Raidicon"]] = {
					type = "toggle",
					name = L["Raidicon"],
					desc = L["Toggle raid icons."],
					get = function() return oRAOMainTank.db.profile.raidicon end,
					set = function(v) oRAOMainTank:ToggleRaidIcon(v) end,
				},

				[L["Growup"]] = {
					type = "toggle",
					name = L["Growup"],
					desc = L["Toggle growup."],
					get = function() return oRAOMainTank.db.profile.growup end,
					set = function(v) oRAOMainTank:ToggleGrowup(v) end,
				},

				[L["Inverse"]] = {
					type = "toggle",
					name = L["Inverse"],
					desc = L["Toggle inverse healthbar."],
					get = function() return oRAOMainTank.db.profile.inverse end,
					set = function(v) oRAOMainTank:ToggleInverse(v) end,
				},

				[L["Deficit"]] = {
					type = "toggle",
					name = L["Deficit"],
					desc = L["Toggle deficit health."],
					get = function() return oRAOMainTank.db.profile.deficit end,
					set = function(v) oRAOMainTank:ToggleDeficit(v) end,
				},
				
				[L["Clickcast"]] = {
					type = "toggle",
					name = L["Clickcast"],
					desc = L["Toggle clickcast support."],
					get = function() return oRAOMainTank.db.profile.clickcast end,
					set = function(v) oRAOMainTank.db.profile.clickcast = v end,
				},
			},
		},	
	}	
}


------------------------------
--      Initialization      --
------------------------------

function oRAOMainTank:OnInitialize()
	self.debugFrame = ChatFrame5
end


function oRAOMainTank:OnEnable()
	self.mtf = {}
	self.mttf = {}
	self.mtttf = {}

	self:SetupFrames()

	self:RegisterEvent("oRA_MainTankUpdate")
	self:RegisterEvent("oRA_LeftRaid")
	self:RegisterEvent("oRA_JoinedRaid", "oRA_MainTankUpdate")
	self:RegisterEvent("RosterLib_RosterChanged", function() self:oRA_MainTankUpdate() end)

	-- Check for Watchdog
	if (WatchDog_OnClick) then
		oRA_MainTankFramesCustomClick = WatchDog_OnClick
	end
end


function oRAOMainTank:OnDisable()
	self:UnregisterAllEvents()
end



------------------------------
--      Event Handlers      --
------------------------------

function oRAOMainTank:oRA_LeftRaid()
	self.mainframe:Hide()
end

function oRAOMainTank:oRA_MainTankUpdate( maintanktable )
	maintanktable = maintanktable or self.core.maintanktable
	if not maintanktable then return end

	local showmt, unitid


	for i = 1, 10 do
		unitid = roster:GetUnitIDFromName(maintanktable[i])

		if unitid then
			if not self.mtf[i] then	self.mtf[i] = self:CreateUnitFrame( self.mainframe, i, "mt" ) end
			if not self.mttf[i] then self.mttf[i] = self:CreateUnitFrame( self.mainframe, i, "mtt" ) end

			self.mtf[i].unit = unitid
			self.mttf[i].unit = unitid

			if self.db.profile.targettarget then
				if not self.mtttf[i] then self.mtttf[i] = self:CreateUnitFrame( self.mainframe, i, "mttt" ) end
				self.mtttf[i].unit = unitid
			end

			showmt = true

		else -- unit nolonger in the raid or unknown
			if self.mtf[i] then self.mtf[i].unit = nil end
			if self.mttf[i] then self.mttf[i].unit = nil end
			if self.mtttf[i] then self.mtttf[i].unit = nil end
		end
	end


	if showmt then
		self.mainframe:Show()
	else
		self.mainframe:Hide()
	end
end



------------------------------
-- ConsoleOption Functions  --
------------------------------

function oRAOMainTank:SetBarTexture(texture)
	if L:HasReverseTranslation(texture) then
		texture = L:GetReverseTranslation(texture)
	else
		texture = "default"
	end
	self.db.profile.bartexture = texture

	for _, f in pairs({ self.mtf, self.mttf, self.mtttf }) do
		for _, f in pairs(f) do
			f.bar:SetStatusBarTexture(bartextures[texture])
			f.bar.texture:SetTexture(bartextures[texture])
		end
	end
end


function oRAOMainTank:SetScale(scale)
	self.db.profile.scale = scale

	self.mainframe:SetScale(scale)
	self:RestorePosition()
end


function oRAOMainTank:SetAlpha(alpha)
	self.db.profile.alpha = alpha

	self.mainframe:SetAlpha(alpha)
end


function oRAOMainTank:ToggleTargetTarget(state)
	self.db.profile.targettarget = state

	if state then
		self:oRA_MainTankUpdate()
	else
		for _, f in pairs(self.mtttf) do
			f:Hide()
		end
	end
end


function oRAOMainTank:ToggleRaidIcon(state)
	self.db.profile.raidicon = state

	if state then return end

	for _, f in pairs({ self.mtf, self.mttf, self.mtttf }) do
		for _, f in pairs(f) do
			f.raidicon:Hide()
		end
	end
end

function oRAOMainTank:ToggleGrowup(state)
	self.db.profile.growup = state


	for _, f in pairs(self.mtf) do
		self:SetStyle(f)
	end

	for _, f in pairs(self.mttf) do
		self:SetStyle(f)
	end

	for _, f in pairs(self.mtttf) do
		self:SetStyle(f)
	end

end

function oRAOMainTank:ToggleInverse(state)
	self.db.profile.inverse = state


        for _, f in pairs(self.mtf) do
		self:UpdateHealthBar(f.bar, self:GetUnit(f))
	end

	for _, f in pairs(self.mttf) do
		self:UpdateHealthBar(f.bar, self:GetUnit(f))
	end

	for _, f in pairs(self.mtttf) do
		self:UpdateHealthBar(f.bar, self:GetUnit(f))
	end

end

function oRAOMainTank:ToggleDeficit(state)
	self.db.profile.deficit = state


        for _, f in pairs(self.mtf) do
		self:UpdateHealthBar(f.bar, self:GetUnit(f))
	end

	for _, f in pairs(self.mttf) do
		self:UpdateHealthBar(f.bar, self:GetUnit(f))
	end

	for _, f in pairs(self.mtttf) do
		self:UpdateHealthBar(f.bar, self:GetUnit(f))
	end

end

------------------------------
--     Utility Functions    --
------------------------------

function oRAOMainTank:SavePosition()
	local f = self.mainframe
	local s = f:GetEffectiveScale()
		
	self.db.profile.posx = f:GetLeft() * s
	self.db.profile.posy = f:GetTop() * s	
end


function oRAOMainTank:RestorePosition()
	local x = self.db.profile.posx
	local y = self.db.profile.posy
		
	if not x or not y then return end
				
	local f = self.mainframe
	local s = f:GetEffectiveScale()

	f:ClearAllPoints()
	f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
end


function oRAOMainTank:SetupFrames()
	local f = CreateFrame("Frame", nil, UIParent)
	f:Hide()
	f:SetMovable(true)
	f:SetScript("OnUpdate", function() self:OnUpdate() end)
	f:SetWidth(100)
	f:SetHeight(100)
	f:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	f:SetAlpha(self.db.profile.alpha)
	f:SetScale(self.db.profile.scale)

	f.update = 0

	self.mainframe = f
	self:RestorePosition()
end


function oRAOMainTank:CreateUnitFrame( parent, id, type )
	-- Main frame
	local f = CreateFrame("Button", "oRA_MainTankFrames_" .. type .. id, parent)
	f:Hide()
	f:EnableMouse(true)
	f:SetMovable(true)
	f:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp", "Button4Up", "Button5Up")
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnEnter", function() self:OnEnter() end)
	f:SetScript("OnLeave", function() GameTooltip:Hide() end)
	f:SetScript("OnClick", function() self:OnClick() end)
	f:SetScript("OnDragStart", function() if IsAltKeyDown() then parent:StartMoving() end end)
	f:SetScript("OnDragStop", function() parent:StopMovingOrSizing() self:SavePosition() end)

	-- Tank Statusbar
	f.bar = CreateFrame("StatusBar", nil, f)
	f.bar:SetMinMaxValues(0,100)

	-- Tank Statusbar background texture, visible when the bar depleats
	f.bar.texture = f.bar:CreateTexture(nil, "BORDER")
	f.bar.texture:SetVertexColor(1, 0, 0, 0.5)

	-- Tank Statusbar text
	f.bar.text = f.bar:CreateFontString(nil, "OVERLAY")
	f.bar.text:SetFontObject(GameFontHighlightSmall)
	f.bar.text:SetJustifyH("RIGHT")

	-- Tank Number
	f.number = f.bar:CreateFontString(nil, "OVERLAY")
	f.number:SetFontObject(GameFontHighlightSmall)
	f.number:SetJustifyH("LEFT")

	-- Tank Name
	f.name = f.bar:CreateFontString(nil, "OVERLAY")
	f.name:SetFontObject(GameFontHighlightSmall)
	f.name:SetJustifyH("LEFT")

	-- Raid Icons
	f.raidicon = f.bar:CreateTexture(nil, "OVERLAY")
	f.raidicon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	f.raidicon:Hide()

	-- Set static stuff and style
	f.number:SetText(id .. ".")
	f.type = type
	f:SetID(id)

	self:SetStyle(f)
	self:SetPosition(f)

	return f
end


function oRAOMainTank:SetStyle(f)
	local relframe, relx, rely
	
	if f.type == "mt" then
		relframe = self.mainframe
		relx = 0
		if self.db.profile.growup then
			rely = 17 * (f:GetID() - 1)
		else 
			rely = -17 * (f:GetID() - 1)
		end
	elseif f.type == "mtt" then
		relframe = self.mtf[f:GetID()]
		relx = 121
		rely = 0
	elseif f.type == "mttt" then
		relframe = self.mttf[f:GetID()]
		relx = 121
		rely = 0
	end

	self:SetWHP(f, 120, 16, "TOPLEFT", relframe, "TOPLEFT", relx , rely)

	f:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = { left = 3, right = 3, top = 3, bottom = 3 } })
	f:SetBackdropColor(0, 0, 0, 1)
	f:SetBackdropBorderColor(1, 1, 1, 0)
	
	f.bar:SetStatusBarTexture(bartextures[self.db.profile.bartexture])
	f.bar.texture:SetTexture(bartextures[self.db.profile.bartexture])

	self:SetWHP(f.bar, f:GetWidth(), f:GetHeight(), "TOPLEFT", f, "TOPLEFT", 0, 0)
	self:SetWHP(f.bar.texture, f.bar:GetWidth(), f.bar:GetHeight(), "CENTER", f.bar, "CENTER", 0, 0)
	self:SetWHP(f.number, 17, f.bar:GetHeight(), "TOPLEFT", f.bar, "TOPLEFT", 0, 0)

	self:SetWHP(f.raidicon, 12, 12, "LEFT", f.number, "RIGHT", 0, 0)

	self:SetWHP(f.name, f.bar:GetWidth() - 60, f.bar:GetHeight(), "LEFT", f.raidicon, "RIGHT", 0, 0)
	self:SetWHP(f.bar.text, 40, f.bar:GetHeight(), "RIGHT", f.bar, "RIGHT", 0, 0)

end


function oRAOMainTank:SetPosition( f )

end


function oRAOMainTank:SetWHP(f, width, height, p1, relative, p2, x, y)
	if not f then return end

	f:SetWidth(width)
	f:SetHeight(height)

	if p1 then
		f:ClearAllPoints()
		f:SetPoint(p1, relative, p2, x, y)
	end
end


function oRAOMainTank:GetUnit(f)
	if not f or not f.type then return end

	if f.type == "mt" then return f.unit end
	if f.type == "mtt" then return f.unit .. "target" end
	if f.type == "mttt" then return f.unit .. "targettarget" end
end


function oRAOMainTank:UpdateFrames(f)
	for _, f in pairs(f) do
		if f.unit then
			local unit = self:GetUnit(f)

			if UnitExists(unit) then
				f.name:SetText(UnitName(unit))

				self:UpdateHealthBar(f.bar, unit)

				if self.db.profile.raidicon then
					self:UpdateRaidIcon(f, unit)
				end

				f:Show()
			else
				f:Hide()
			end

		else
			f:Hide()
		end
	end
end


function oRAOMainTank:UpdateHealthBar(bar, unit)
	local cur, max = UnitHealth(unit) or 0, UnitHealthMax(unit) or 0
	local perc = cur / max

	bar:SetMinMaxValues(0, max)
	
	if self.db.profile.inverse then
		bar:SetValue(max - cur)
		bar.texture:SetVertexColor(0, 1, 0, 0.5)
	else
		bar:SetValue(cur)
		bar.texture:SetVertexColor(1, 0, 0, 0.5)
	end
	bar:SetStatusBarColor(self:GetHealthBarColor(perc))
	
	if self.db.profile.deficit then
		local val = max - cur
		if val > 1000 then
			val = ceil(val/100)/10 .. "k"
		elseif val == 0 then
			val = ""
		end
		
		bar.text:SetText(val)
	else
		bar.text:SetText(ceil(perc * 100) .. "%")
	end

	bar:Show()
end


function oRAOMainTank:GetHealthBarColor(perc)
	local r, g

	if perc > 0.5 then
		r = (1.0 - perc) * 2
		g = 1.0
	else
		r = 1.0
		g = perc * 2
	end

	return r, g, 0
end


function oRAOMainTank:UpdateRaidIcon(f, unit)
	local icon = GetRaidTargetIndex(unit)

	if icon then
		SetRaidTargetIconTexture(f.raidicon, icon)
		f.raidicon:Show()
	else
		f.raidicon:Hide()
	end
end


-------------------------------
--   Frame Script Functions  --
-------------------------------

function oRAOMainTank:OnUpdate()
	this.update = this.update + arg1

	if this.update >= 0.5 then
		self:UpdateFrames(self.mtf)
		self:UpdateFrames(self.mttf)

		if self.db.profile.targettarget then
			self:UpdateFrames(self.mtttf)
		end

		this.update = 0
	end
end


function oRAOMainTank:OnEnter()
	local unit = self:GetUnit(this)

	GameTooltip_SetDefaultAnchor(GameTooltip, this)

	if unit and GameTooltip:SetUnit(unit) then
		this.updateTooltip = TOOLTIP_UPDATE_TIME
	else
		this.updateTooltip = nil
	end
end


function oRAOMainTank:OnClick()
	local unit = self:GetUnit(this)

	if self.db.profile.clickcast and oRA_MainTankFramesCustomClick then
		oRA_MainTankFramesCustomClick(arg1, unit)
	elseif UnitExists(unit) then
		TargetUnit(unit)
	end
end
